// ------------------------------------------------------------------------
// eca-qtwaveform.cpp: Qt-widget for visualizing audio signals.
// Copyright (C) 1999 Kai Vehmanen (kaiv@wakkanet.fi)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
// ------------------------------------------------------------------------

#include <string>
#include <vector>
#include <cmath>

#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qprogressdialog.h>
#include <qlabel.h>
#include <qpushbutton.h>

#include <kvutils.h>

#include "audioio.h"
#include "eca-qtwaveform.h"
#include "debug.h"

QEWaveForm::QEWaveForm(AUDIO_IO_DEVICE* iod, QWidget *parent=0, const char *name=0 )
        : QWidget( parent, name )
{
  startTimer(250);
  //  setMaximumHeight(100);
  setMinimumSize(300, 100);
  iodevice = iod;
  xposcoord = 0;

  string caption = "qtEcasound - Wave view (" + iodevice->label() + ")";
  setCaption(caption.c_str());

  if (iodevice->is_realtime() == true ||
      iodevice->length_in_samples() == 0 ||
      iodevice->io_mode() == si_write) 
    valid_iodevice = false;
  else 
    valid_iodevice = true;
}

void QEWaveForm::timerEvent( QTimerEvent * ) {
  newxposcoord = (int)((double)iodevice->position_in_samples() / iodevice->length_in_samples() * width());
  if (newxposcoord != xposcoord) {
    static QPainter p;
    p.begin(this);
    p.setPen(backgroundColor());
    p.drawLine(xposcoord, 0, xposcoord, height());
    p.setPen(QColor("steel blue"));
    ycoord = height() / 2 / SAMPLE_BUFFER::ch_count;
    for(char ch = 0; ch < SAMPLE_BUFFER::ch_count; ch++) {
      ycoord += (height() / SAMPLE_BUFFER::ch_count) * ch;      
      p.drawLine(xposcoord, ycoord - (int)(waveblocks[xposcoord * step].min[ch] / 32767.0 * waveheight),
		 xposcoord, ycoord - (int)(waveblocks[xposcoord * step].max[ch] / 32767.0 * waveheight));
    }
    xposcoord = newxposcoord;
    p.setPen(QColor(black));
    p.drawLine(xposcoord, 0, xposcoord, height());
    p.end();
    //    repaint(true);
  }
}

void QEWaveForm::updateWaveData(void)
{
  if (!valid_iodevice) return;

  //  emit setTotalSteps(iodevice->length_in_seconds());
  //  repaint(true);

  //  QProgressDialog progress ("Analyzing wave data...","Cancel",iodevice->length_in_seconds(),0,0,true);
  //  QProgressDialog* progress = new QProgressDialog();
  //  progress->setLabel(new QLabel("Analyzing wave data...",progress,"qlp"));
  //  progress->setCancelButton(new
  //  QPushButton("Cancel",progress,"qcp"));
  //  progress->setMinimumDuration(0);
  //  progress.setProgress(0);
  //  progress.show();
  //  progress->setTotalSteps(ecaparams->inputs[0]->length_in_seconds());
  //  progress->setTotalSteps(iodevice->length_in_seconds());
  //  progress->repaint();

  long save_pos = iodevice->position_in_samples();
  iodevice->first();

  SAMPLE_BUFFER* t = new SAMPLE_BUFFER (QEWaveForm::buffersize);
  QEWaveBlock blocktmp;
  //  waveblocks.resize((int)(iodevice->length_in_samples() / QEWaveForm::buffersize));

  vector<QEWaveBlock>::size_type bnum = 0;
  while(!iodevice->finished()) {
    iodevice->get_sample(t);
    //    emit setProgress(iodevice->position_in_seconds());   
    //    progress.setProgress(iodevice->position_in_seconds());
    // progress->repaint();
    //   if (progress.wasCancelled()) break;

    for(char ch = 0; ch < SAMPLE_BUFFER::ch_count; ch++) {

      blocktmp.max[ch] = (int)(32767.0 * t->max_value(ch) / SAMPLE_BUFFER::max_amplitude);
      //      cerr << blocktmp.max[ch];
      //      cerr << "-";
      blocktmp.min[ch] = (int)(32767.0 * t->min_value(ch) / SAMPLE_BUFFER::max_amplitude);
      //      cerr << blocktmp.min[ch];
      //      cerr << "<br>\n";
    }

    waveblocks.push_back(blocktmp);
    //    waveblocks[bnum] = blocktmp;
    ++bnum;
    if (bnum == iodevice->length_in_samples() / QEWaveForm::buffersize) break;
  }

  iodevice->position_in_samples(save_pos);
}

void QEWaveForm::paintEvent( QPaintEvent* e )
{
  static QRect ur;
  ur = e->rect(); 
  static QPixmap pix;
  pix.resize(ur.size());
  pix.fill( this, ur.topLeft() );     // fill with widget background
  static QPainter p;
  p.begin( &pix );
  p.setPen(QColor("steel blue"));

  samples_per_pixel = iodevice->length_in_samples() / width();
  ecadebug->msg(2,"eca-qtwaveform::drawWaveform(), samples_per_pixel " + kvu_numtostr(samples_per_pixel));

  step = samples_per_pixel / QEWaveForm::buffersize;
  if (step == 0) step == 1;
  
  xcoord = 0;
  waveheight = height() / SAMPLE_BUFFER::ch_count / 2 - 5;  
  t = 0;
  while (t < waveblocks.size()) {
    ycoord = height() / 2 / SAMPLE_BUFFER::ch_count;
    for(char ch = 0; ch < SAMPLE_BUFFER::ch_count; ch++) {
      ycoord += (height() / SAMPLE_BUFFER::ch_count) * ch;
      p.drawLine(xcoord, ycoord - (int)(waveblocks[t].min[ch] / 32767.0 * waveheight),
	       xcoord, ycoord - (int)(waveblocks[t].max[ch] / 32767.0 * waveheight));
    }
    t += step;
    ++xcoord;
  }

  p.setPen(QColor(black));
  p.drawLine(xposcoord, 0, xposcoord, height());
  
  p.end();
  bitBlt( this, ur.topLeft(), &pix );
}
